@@ -46,7 +46,7 @@ pub fn true_eq<A: Prop>(a: A) -> Eq<A, True> {
     (True.map_any(), Rc::new(move |_| a.clone()))
 }
 
-/// `(a = b) => (¬a = ¬b)`
+/// `(a = b) => (¬b = ¬a)`
 pub fn modus_tollens<A: Prop, B: Prop>((f0, f1): Eq<A, B>) -> Eq<Not<B>, Not<A>> {
     let f02 = imply::modus_tollens(f0);
     let f12 = imply::modus_tollens(f1);
@@ -76,11 +76,19 @@ pub fn to_eq_false<A: Prop>(n_a: Not<A>) -> Eq<A, False> {
 }
 
 /// `¬(a = b) ∧ a  =>  ¬b`.
-pub fn contra<A: DProp, B: DProp>(f: Not<Eq<A, B>>, a: A) -> Not<B> {
-    match (A::decide(), B::decide()) {
-        (Left(a), Left(b)) => match f(and::to_eq_pos((a, b))) {},
-        (_, Right(not_b)) => not_b,
-        (Right(not_a), _) => match not_a(a) {},
+pub fn contra<A: Prop, B: DProp>(f: Not<Eq<A, B>>, a: A) -> Not<B> {
+    contra_excm(f, a, B::decide())
+}
+
+/// `¬(a = b) ∧ a  =>  ¬b`.
+pub fn contra_excm<A: Prop, B: Prop>(
+    f: Not<Eq<A, B>>,
+    a: A,
+    excm_b: ExcM<B>
+) -> Not<B> {
+    match excm_b {
+        Left(b) => not::absurd(f, and::to_eq_pos((a, b))),
+        Right(not_b) => not_b,
     }
 }
 
