@@ -41,6 +41,28 @@ pub fn ty_in_right_arg<A: Prop, B: Prop, C: Prop>((ab, pord): Ty<A, B>, eq: Eq<B
     (imply::in_right_arg(ab, eq.clone()), pord.by_eq_right(eq))
 }
 
+/// `(x : false) => ¬x`.
+pub fn ty_not<X: Prop>(ty_x_false: Ty<X, False>) -> Not<X> {ty_x_false.0}
+
+/// `(x : a) ⋀ a => (x : true)`.
+pub fn ty_triv<X: Prop, A: Prop>(
+    ty_x_a: Ty<X, A>,
+    a: A,
+) -> Ty<X, True> {
+    let eq_a_true = (True.map_any(), a.map_any());
+    ty_in_right_arg(ty_x_a, eq_a_true)
+}
+
+/// `(x : a) ⋀ ¬a => (x : false)`.
+pub fn ty_non_triv<X: Prop, A: Prop>(
+    ty_x_a: Ty<X, A>,
+    na: Not<A>,
+) -> Ty<X, False> {
+    let eq_a_false: Eq<A, False> =
+        (Rc::new(move |a| na(a)), Rc::new(move |fa| imply::absurd()(fa)));
+    ty_in_right_arg(ty_x_a, eq_a_false)
+}
+
 /// `true == ltrue`.
 pub fn eq_true_ltrue<N: Nat>() -> Eq<True, LTrue<N>> {
     (LTrue(Default::default()).map_any(), True.map_any())
