@@ -326,6 +326,24 @@ pub fn set_h2<X: LProp, Y: LProp, X2: LProp, Y2: LProp>(
     quality::to_eq(q_tr_q_x2_y2).0(Left(True))
 }
 
+/// `(x : set) ⋀ (y : set) => ((x ⋀ y) : set)`
+pub fn set_and<X: LProp, Y: LProp>(
+    ty_x: Ty<X, Set>,
+    ty_y: Ty<Y, Set>
+) -> Ty<And<X, Y>, Set> {
+    let eq: Eq<And<Set, Set>, Set> = (Set.map_any(),(Set, Set).map_any());
+    path_semantics::ty_in_right_arg(path_semantics::ty_and(ty_x, ty_y), eq)
+}
+
+/// `(x : set) ⋀ (y : set) => ((x ⋁ y) : set)`.
+pub fn set_or<X: LProp, Y: LProp>(
+    ty_x: Ty<X, Set>,
+    ty_y: Ty<Y, Set>
+) -> Ty<Or<X, Y>, Set> {
+    let eq: Eq<Or<Set, Set>, Set> = (Set.map_any(), Left(Set).map_any());
+    path_semantics::ty_in_right_arg(path_semantics::ty_or(ty_x, ty_y), eq)
+}
+
 /// `(x : a) ⋀ (x : b)  =>  (a ~~ b)` when `a` and `b` are homotopy level 0.
 pub fn h0_ext<A: HProp<Z>, B: HProp<Z>, X: LProp>(
     ty_xa: Ty<X, A>,
@@ -391,7 +409,7 @@ pub fn h1_false<X: LProp, N: Nat, A: HProp<S<N>>>(
 }
 
 /// `(x : a) ⋀ (x : true) ⋀ ((x ~~ x) == x)  =>  a`.
-pub fn h1_true<X: LProp, A: HProp<S<Z>>>(
+pub fn h1_true<X: LProp, N: Nat, A: HProp<S<N>>>(
     ty_x_a: Ty<X, A>,
     ty_x_true: Ty<X, True>,
     q: Eq<Q<X, X>, X>,
@@ -410,6 +428,11 @@ pub fn h0_true_q<X: LProp>(ty_x: Ty<X, True>) -> Q<X, X> {
     quality::transitivity(quality::symmetry(f.clone()), f)
 }
 
+/// `(x : false)  =>  (x ~~ x)`.
+pub fn h1_false_q<X: LProp>(ty_x_false: Ty<X, False>) -> Q<X, X> {
+    quality::to_eq(<False as HomotopyLevel<S<Z>>>::hn(ty_x_false.clone(), ty_x_false)).0(True)
+}
+
 /// `(x : true) => x`.
 pub fn h0_lproof<X: LProp>(ty_x: Ty<X, True>) -> X {
     quality::to_eq(h0_q_true(ty_x)).0(True)
@@ -443,3 +466,49 @@ pub fn contradict<X: LProp>(
 ) -> False {
     quality::to_eq(q_contradict(ty_x_true, ty_x_false)).0(True)
 }
+
+/// `(x : a) ⋀ ((x ~~ x) == x)  =>  (a ⋁ ¬a) == ((x : true) ⋁ (x : false))`.
+pub fn h1_lim_excm<X: LProp, N: Nat, A: HProp<S<N>>>(
+    ty_x_a: Ty<X, A>,
+    lim: Eq<Q<X, X>, X>,
+) -> Eq<ExcM<A>, Or<Ty<X, True>, Ty<X, False>>> {
+    let ty_x_a_clone = ty_x_a.clone();
+    (
+        Rc::new(move |excm| match excm {
+            Left(a) => Left(path_semantics::ty_triv(ty_x_a_clone.clone(), a)),
+            Right(na) => Right(path_semantics::ty_non_triv(ty_x_a_clone.clone(), na)),
+        }),
+        Rc::new(move |or| match or {
+            Left(ty_x_true) => Left(h1_true(ty_x_a.clone(), ty_x_true, lim.clone())),
+            Right(ty_x_false) => Right(h1_false(ty_x_a.clone(), ty_x_false, lim.clone())),
+        })
+    )
+}
+
+/// `(x : a) ⋀ ((x ~~ x) == x)  =>  (x : true) ⋁ (x : false)`.
+pub fn h1_lim_excm_da<X: LProp, N: Nat, A: DProp + HProp<S<N>>>(
+    ty_x_a: Ty<X, A>,
+    lim: Eq<Q<X, X>, X>,
+) -> Or<Ty<X, True>, Ty<X, False>> {
+    h1_lim_excm(ty_x_a, lim).0(A::decide())
+}
+
+/// `(x : false) ⋀ ((x ~~ x) == x)  =>  false`.
+pub fn h1_false_lim_contradict<X: LProp>(
+    ty_x_false: Ty<X, False>,
+    lim: Eq<Q<X, X>, X>
+) -> False {
+    ty_x_false.0(lim.0(h1_false_q(ty_x_false.clone())))
+}
+
+/// `(x : a) ⋀ ((x ~~ x) == x) ⋀ (a ⋁ ¬a)  =>  (x : true)`.
+pub fn h1_lim_excm_true<X: LProp, N: Nat, A: HProp<S<N>>>(
+    ty_x_a: Ty<X, A>,
+    lim: Eq<Q<X, X>, X>,
+    excm: Or<A, Not<A>>,
+) -> Ty<X, True> {
+    match h1_lim_excm(ty_x_a, lim.clone()).0(excm) {
+        Left(ty_x_true) => ty_x_true,
+        Right(ty_x_false) => imply::absurd()(h1_false_lim_contradict(ty_x_false, lim)),
+    }
+}
